import { FormGroup, FormArray } from '@angular/forms';
import { from, of, forkJoin } from 'rxjs';
import { map } from 'rxjs/operators';

function mergeError(errorArray) {
    const sanitizedErrors = errorArray === null || errorArray === void 0 ? void 0 : errorArray.filter(({ err }) => Object.keys(err || {}).length);
    return !(sanitizedErrors === null || sanitizedErrors === void 0 ? void 0 : sanitizedErrors.length) ? null
        : Object.assign({}, ...sanitizedErrors.map(({ key, err }) => ({ [key]: err })));
}
function loopControl(control, selector) {
    if (control instanceof FormGroup) {
        return selector(Object.entries(control.controls)
            .map(([key, ctrl]) => ({ key, control: ctrl })));
    }
    else if (control instanceof FormArray) {
        return selector(control.controls
            .map((ctrl, key) => ({ key, control: ctrl })));
    }
}
function compositeValidator() {
    const validator = (ctrl) => {
        const vl = ctrl === null || ctrl === void 0 ? void 0 : ctrl.validator;
        if (typeof vl === 'function') {
            return vl(ctrl);
        }
        const errAry = loopControl(ctrl, pairs => pairs.reduce((acc, { key, control }) => [...acc, { key, err: validator(control) }], []));
        return mergeError(errAry);
    };
    return control => {
        if (control instanceof FormGroup || control instanceof FormArray) {
            return validator(control);
        }
    };
}
function compositeAsyncValidator() {
    const validator = (ctrl) => {
        const vl = ctrl === null || ctrl === void 0 ? void 0 : ctrl.asyncValidator;
        if (typeof vl === 'function') {
            return vl(ctrl);
        }
        const asyncErrAry = loopControl(ctrl, pairs => pairs.reduce((acc, { key, control }) => {
            const asyncErr = validator(control);
            if (asyncErr) {
                return [...acc, from(asyncErr).pipe(map(err => ({ key, err })))];
            }
            return acc;
        }, []));
        return !(asyncErrAry === null || asyncErrAry === void 0 ? void 0 : asyncErrAry.length) ? of(null)
            : forkJoin(asyncErrAry).pipe(map(errAry => mergeError(errAry)));
    };
    return control => {
        if (control instanceof FormGroup || control instanceof FormArray) {
            return validator(control);
        }
    };
}

/**
 * Generated bundle index. Do not edit.
 */

export { compositeAsyncValidator, compositeValidator };
//# sourceMappingURL=ngx-reactive-form-utils-composite-validator.js.map
