export * from 'ngx-reactive-form/utils/composite-validator';
export * from 'ngx-reactive-form/utils/deferred-value-accessor';
export * from 'ngx-reactive-form/utils/provide-value-accessor';
import { FormGroup, FormArray } from '@angular/forms';

function lazy(factory, destroy) {
    let init = false;
    let lVal;
    let err;
    function value() {
        if (!init) {
            init = true;
            try {
                lVal = factory();
            }
            catch (e) {
                err = e;
            }
        }
        if (err) {
            throw err;
        }
        return lVal;
    }
    value.reset = () => {
        if (init) {
            err = undefined;
            if (typeof destroy === 'function') {
                destroy(lVal);
            }
            init = false;
        }
    };
    value.isInit = () => init;
    return value;
}

function getElements(ctrls) {
    return Array.isArray(ctrls) ? ctrls :
        typeof ctrls === 'object' && Object.values(ctrls) || [];
}
function tapControls(root, action, updateState) {
    for (const ctrl of getElements(root && root.controls)) {
        if (ctrl instanceof FormGroup || ctrl instanceof FormArray) {
            tapControls(ctrl, action, updateState);
            continue;
        }
        if (typeof action === 'function') {
            action(ctrl);
        }
        if (updateState) {
            ctrl.updateValueAndValidity();
        }
    }
}

function traverseControl(control, ...ps) {
    if (ps.length === 0 || !control) {
        return control;
    }
    if (control instanceof FormGroup || control instanceof FormArray) {
        const [p, ...lps] = ps;
        return traverseControl(control.controls[p], ...lps);
    }
}

function uniq(items) {
    return (items || []).filter((val, idx, self) => val && self.indexOf(val) === idx);
}

/**
 * Generated bundle index. Do not edit.
 */

export { lazy, tapControls, traverseControl, uniq };
//# sourceMappingURL=ngx-reactive-form-utils.js.map
